/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class gnu_java_net_VMPlainDatagramSocketImpl */

#ifndef _Included_gnu_java_net_VMPlainDatagramSocketImpl
#define _Included_gnu_java_net_VMPlainDatagramSocketImpl
#ifdef __cplusplus
extern "C" {
#endif
#undef gnu_java_net_VMPlainDatagramSocketImpl_IP_TTL
#define gnu_java_net_VMPlainDatagramSocketImpl_IP_TTL 7777L
/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    bind
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;ILjava/net/InetAddress;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_bind
  (JNIEnv *, jclass, jobject, jint, jobject);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    create
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_create
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    connect
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;Ljava/net/InetAddress;I)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_connect
  (JNIEnv *, jclass, jobject, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    nativeSendTo
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;Ljava/net/InetAddress;I[BII)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_nativeSendTo
  (JNIEnv *, jclass, jobject, jobject, jint, jbyteArray, jint, jint);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    nativeReceive
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;[BII[B[I[I)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_nativeReceive
  (JNIEnv *, jclass, jobject, jbyteArray, jint, jint, jbyteArray, jintArray, jintArray);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    setOption
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;ILjava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_setOption
  (JNIEnv *, jclass, jobject, jint, jobject);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    getOption
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;I)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_getOption
  (JNIEnv *, jclass, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    close
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_close
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    join
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;Ljava/net/InetAddress;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_join
  (JNIEnv *, jclass, jobject, jobject);

/*
 * Class:     gnu_java_net_VMPlainDatagramSocketImpl
 * Method:    leave
 * Signature: (Lgnu/java/net/PlainDatagramSocketImpl;Ljava/net/InetAddress;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainDatagramSocketImpl_leave
  (JNIEnv *, jclass, jobject, jobject);

#ifdef __cplusplus
}
#endif
#endif
