/* DO NOT EDIT THIS FILE - it is machine generated */

#ifndef __gnu_xml_libxmlj_dom_GnomeXPathNSResolver__
#define __gnu_xml_libxmlj_dom_GnomeXPathNSResolver__

#include <jni.h>

#ifdef __cplusplus
extern "C"
{
#endif


#ifdef __cplusplus
}
#endif

#endif /* __gnu_xml_libxmlj_dom_GnomeXPathNSResolver__ */
