/* DO NOT EDIT THIS FILE - it is machine generated */
#include <jni.h>
/* Header for class gnu_java_net_VMPlainSocketImpl */

#ifndef _Included_gnu_java_net_VMPlainSocketImpl
#define _Included_gnu_java_net_VMPlainSocketImpl
#ifdef __cplusplus
extern "C" {
#endif
/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    setOption
 * Signature: (Lgnu/java/net/PlainSocketImpl;ILjava/lang/Object;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_setOption
  (JNIEnv *, jclass, jobject, jint, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    getOption
 * Signature: (Lgnu/java/net/PlainSocketImpl;I)Ljava/lang/Object;
 */
JNIEXPORT jobject JNICALL Java_gnu_java_net_VMPlainSocketImpl_getOption
  (JNIEnv *, jclass, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    create
 * Signature: (Lgnu/java/net/PlainSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_create
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    connect
 * Signature: (Lgnu/java/net/PlainSocketImpl;Ljava/net/InetAddress;I)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_connect
  (JNIEnv *, jclass, jobject, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    bind
 * Signature: (Lgnu/java/net/PlainSocketImpl;Ljava/net/InetAddress;I)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_bind
  (JNIEnv *, jclass, jobject, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    listen
 * Signature: (Lgnu/java/net/PlainSocketImpl;I)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_listen
  (JNIEnv *, jclass, jobject, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    accept
 * Signature: (Lgnu/java/net/PlainSocketImpl;Ljava/net/SocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_accept
  (JNIEnv *, jclass, jobject, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    available
 * Signature: (Lgnu/java/net/PlainSocketImpl;)I
 */
JNIEXPORT jint JNICALL Java_gnu_java_net_VMPlainSocketImpl_available
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    close
 * Signature: (Lgnu/java/net/PlainSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_close
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    read
 * Signature: (Lgnu/java/net/PlainSocketImpl;[BII)I
 */
JNIEXPORT jint JNICALL Java_gnu_java_net_VMPlainSocketImpl_read
  (JNIEnv *, jclass, jobject, jbyteArray, jint, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    write
 * Signature: (Lgnu/java/net/PlainSocketImpl;[BII)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_write
  (JNIEnv *, jclass, jobject, jbyteArray, jint, jint);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    shutdownInput
 * Signature: (Lgnu/java/net/PlainSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_shutdownInput
  (JNIEnv *, jclass, jobject);

/*
 * Class:     gnu_java_net_VMPlainSocketImpl
 * Method:    shutdownOutput
 * Signature: (Lgnu/java/net/PlainSocketImpl;)V
 */
JNIEXPORT void JNICALL Java_gnu_java_net_VMPlainSocketImpl_shutdownOutput
  (JNIEnv *, jclass, jobject);

#ifdef __cplusplus
}
#endif
#endif
